@@ -4,9 +4,9 @@ Implementation of a Threaded Modbus Server
 
 '''
 from binascii import b2a_hex
-import SocketServer
 import serial
 import socket
+import traceback
 
 from pymodbus.constants import Defaults
 from pymodbus.factory import ServerDecoder
@@ -14,8 +14,9 @@ from pymodbus.datastore import ModbusServerContext
 from pymodbus.device import ModbusControlBlock
 from pymodbus.device import ModbusDeviceIdentification
 from pymodbus.transaction import *
-from pymodbus.exceptions import NotImplementedException
+from pymodbus.exceptions import NotImplementedException, NoSuchSlaveException
 from pymodbus.pdu import ModbusExceptions as merror
+from pymodbus.compat import socketserver, byte2int
 
 #---------------------------------------------------------------------------#
 # Logging
@@ -27,7 +28,7 @@ _logger = logging.getLogger(__name__)
 #---------------------------------------------------------------------------#
 # Protocol Handlers
 #---------------------------------------------------------------------------#
-class ModbusBaseRequestHandler(SocketServer.BaseRequestHandler):
+class ModbusBaseRequestHandler(socketserver.BaseRequestHandler):
     ''' Implements the modbus server protocol
 
     This uses the socketserver.BaseRequestHandler to implement
@@ -56,8 +57,13 @@ class ModbusBaseRequestHandler(SocketServer.BaseRequestHandler):
         try:
             context = self.server.context[request.unit_id]
             response = request.execute(context)
-        except Exception, ex:
-            _logger.debug("Datastore unable to fulfill request: %s" % ex)
+        except NoSuchSlaveException as ex:
+            _logger.debug("requested slave does not exist: %s" % request.unit_id )
+            if self.server.ignore_missing_slaves:
+                return # the client will simply timeout waiting for a response
+            response = request.doException(merror.GatewayNoResponse)
+        except Exception as ex:
+            _logger.debug("Datastore unable to fulfill request: %s; %s", ex, traceback.format_exc() )
             response = request.doException(merror.SlaveFailure)
         response.transaction_id = request.transaction_id
         response.unit_id = request.unit_id
@@ -79,6 +85,34 @@ class ModbusBaseRequestHandler(SocketServer.BaseRequestHandler):
         raise NotImplementedException("Method not implemented by derived class")
 
 
+# class ModbusSingleRequestHandler(ModbusBaseRequestHandler):
+#     ''' Implements the modbus server protocol
+#
+#     This uses the socketserver.BaseRequestHandler to implement
+#     the client handler for a single client(serial clients)
+#     '''
+#
+#     def handle(self):
+#         ''' Callback when we receive any data
+#         '''
+#         while self.running:
+#             try:
+#                 data = self.request.recv(1024)
+#                 if data:
+#                     if _logger.isEnabledFor(logging.DEBUG):
+#                         _logger.debug(' '.join([hex(byte2int(x)) for x in data]))
+#                     self.framer.processIncomingPacket(data, self.execute)
+#             except Exception as msg:
+#                 # since we only have a single socket, we cannot exit
+#                 _logger.error("Socket error occurred %s" % msg)
+#
+#     def send(self, message):
+#         ''' Send a request (string) to the network
+#
+#         :param message: The unencoded modbus response
+#         '''
+
+
 class ModbusSingleRequestHandler(ModbusBaseRequestHandler):
     ''' Implements the modbus server protocol
 
@@ -94,9 +128,9 @@ class ModbusSingleRequestHandler(ModbusBaseRequestHandler):
                 data = self.request.recv(1024)
                 if data:
                     if _logger.isEnabledFor(logging.DEBUG):
-                        _logger.debug(" ".join([hex(ord(x)) for x in data]))
+                        _logger.debug(" ".join([hex(byte2int(x)) for x in data]))
                     self.framer.processIncomingPacket(data, self.execute)
-            except Exception, msg:
+            except Exception as msg:
                 # since we only have a single socket, we cannot exit
                 _logger.error("Socket error occurred %s" % msg)
 
@@ -121,21 +155,36 @@ class ModbusConnectedRequestHandler(ModbusBaseRequestHandler):
     '''
 
     def handle(self):
-        ''' Callback when we receive any data
+        '''Callback when we receive any data, until self.running becomes not True.  Blocks indefinitely
+        awaiting data.  If shutdown is required, then the global socket.settimeout(<seconds>) may be
+        used, to allow timely checking of self.running.  However, since this also affects socket
+        connects, if there are outgoing socket connections used in the same program, then these will
+        be prevented, if the specfied timeout is too short.  Hence, this is unreliable.
+
+        To respond to Modbus...Server.server_close() (which clears each handler's self.running),
+        derive from this class to provide an alternative handler that awakens from time to time when
+        no input is available and checks self.running.  Use Modbus...Server( handler=... ) keyword
+        to supply the alternative request handler class.
+
         '''
         while self.running:
             try:
                 data = self.request.recv(1024)
                 if not data: self.running = False
                 if _logger.isEnabledFor(logging.DEBUG):
-                    _logger.debug(" ".join([hex(ord(x)) for x in data]))
+                    _logger.debug(' '.join([hex(byte2int(x)) for x in data]))
                 # if not self.server.control.ListenOnly:
                 self.framer.processIncomingPacket(data, self.execute)
-            except socket.timeout: pass
-            except socket.error, msg:
+            except socket.timeout as msg:
+                if _logger.isEnabledFor(logging.DEBUG):
+                    _logger.debug("Socket timeout occurred %s", msg)
+                pass
+            except socket.error as msg:
                 _logger.error("Socket error occurred %s" % msg)
                 self.running = False
-            except: self.running = False
+            except:
+                _logger.error("Socket exception occurred %s" % traceback.format_exc() )
+                self.running = False
 
     def send(self, message):
         ''' Send a request (string) to the network
@@ -167,11 +216,11 @@ class ModbusDisconnectedRequestHandler(ModbusBaseRequestHandler):
                 data, self.request = self.request
                 if not data: self.running = False
                 if _logger.isEnabledFor(logging.DEBUG):
-                    _logger.debug(" ".join([hex(ord(x)) for x in data]))
+                    _logger.debug(' '.join([hex(byte2int(x)) for x in data]))
                 # if not self.server.control.ListenOnly:
                 self.framer.processIncomingPacket(data, self.execute)
             except socket.timeout: pass
-            except socket.error, msg:
+            except socket.error as msg:
                 _logger.error("Socket error occurred %s" % msg)
                 self.running = False
             except: self.running = False
@@ -192,7 +241,7 @@ class ModbusDisconnectedRequestHandler(ModbusBaseRequestHandler):
 #---------------------------------------------------------------------------#
 # Server Implementations
 #---------------------------------------------------------------------------#
-class ModbusTcpServer(SocketServer.ThreadingTCPServer):
+class ModbusTcpServer(socketserver.ThreadingTCPServer):
     '''
     A modbus threaded tcp socket server
 
@@ -201,7 +250,7 @@ class ModbusTcpServer(SocketServer.ThreadingTCPServer):
     server context instance.
     '''
 
-    def __init__(self, context, framer=None, identity=None, address=None):
+    def __init__(self, context, framer=None, identity=None, address=None, handler=None, **kwargs):
         ''' Overloaded initializer for the socket server
 
         If the identify structure is not passed in, the ModbusControlBlock
@@ -211,6 +260,8 @@ class ModbusTcpServer(SocketServer.ThreadingTCPServer):
         :param framer: The framer strategy to use
         :param identity: An optional identify structure
         :param address: An optional (interface, port) to bind to.
+        :param handler: A handler for each client session; default is ModbusConnectedRequestHandler
+        :param ignore_missing_slaves: True to not send errors on a request to a missing slave
         '''
         self.threads = []
         self.decoder = ServerDecoder()
@@ -218,11 +269,12 @@ class ModbusTcpServer(SocketServer.ThreadingTCPServer):
         self.context = context or ModbusServerContext()
         self.control = ModbusControlBlock()
         self.address = address or ("", Defaults.Port)
+        self.ignore_missing_slaves = kwargs.get('ignore_missing_slaves', Defaults.IgnoreMissingSlaves)
 
         if isinstance(identity, ModbusDeviceIdentification):
             self.control.Identity.update(identity)
 
-        SocketServer.ThreadingTCPServer.__init__(self,
+        socketserver.ThreadingTCPServer.__init__(self,
             self.address, ModbusConnectedRequestHandler)
 
     def process_request(self, request, client):
@@ -232,7 +284,16 @@ class ModbusTcpServer(SocketServer.ThreadingTCPServer):
         :param client: The address of the client
         '''
         _logger.debug("Started thread to serve client at " + str(client))
-        SocketServer.ThreadingTCPServer.process_request(self, request, client)
+        socketserver.ThreadingTCPServer.process_request(self, request, client)
+
+    def shutdown(self):
+        ''' Stops the serve_forever loop.
+
+        Overridden to signal handlers to stop.
+        '''
+        for thread in self.threads:
+            thread.running = False
+        socketserver.ThreadingTCPServer.shutdown(self)
 
     def server_close(self):
         ''' Callback for stopping the running server
@@ -243,7 +304,7 @@ class ModbusTcpServer(SocketServer.ThreadingTCPServer):
             thread.running = False
 
 
-class ModbusUdpServer(SocketServer.ThreadingUDPServer):
+class ModbusUdpServer(socketserver.ThreadingUDPServer):
     '''
     A modbus threaded udp socket server
 
@@ -252,7 +313,7 @@ class ModbusUdpServer(SocketServer.ThreadingUDPServer):
     server context instance.
     '''
 
-    def __init__(self, context, framer=None, identity=None, address=None):
+    def __init__(self, context, framer=None, identity=None, address=None, handler=None, **kwargs):
         ''' Overloaded initializer for the socket server
 
         If the identify structure is not passed in, the ModbusControlBlock
@@ -262,6 +323,8 @@ class ModbusUdpServer(SocketServer.ThreadingUDPServer):
         :param framer: The framer strategy to use
         :param identity: An optional identify structure
         :param address: An optional (interface, port) to bind to.
+        :param handler: A handler for each client session; default is ModbusDisonnectedRequestHandler
+        :param ignore_missing_slaves: True to not send errors on a request to a missing slave
         '''
         self.threads = []
         self.decoder = ServerDecoder()
@@ -269,11 +332,12 @@ class ModbusUdpServer(SocketServer.ThreadingUDPServer):
         self.context = context or ModbusServerContext()
         self.control = ModbusControlBlock()
         self.address = address or ("", Defaults.Port)
+        self.ignore_missing_slaves = kwargs.get('ignore_missing_slaves', Defaults.IgnoreMissingSlaves)
 
         if isinstance(identity, ModbusDeviceIdentification):
             self.control.Identity.update(identity)
 
-        SocketServer.ThreadingUDPServer.__init__(self,
+        socketserver.ThreadingUDPServer.__init__(self,
             self.address, ModbusDisconnectedRequestHandler)
 
     def process_request(self, request, client):
@@ -284,7 +348,7 @@ class ModbusUdpServer(SocketServer.ThreadingUDPServer):
         '''
         packet, socket = request # TODO I might have to rewrite
         _logger.debug("Started thread to serve client at " + str(client))
-        SocketServer.ThreadingUDPServer.process_request(self, request, client)
+        socketserver.ThreadingUDPServer.process_request(self, request, client)
 
     def server_close(self):
         ''' Callback for stopping the running server
@@ -297,7 +361,7 @@ class ModbusUdpServer(SocketServer.ThreadingUDPServer):
 
 class ModbusSerialServer(object):
     '''
-    A modbus threaded udp socket server
+    A modbus threaded serial socket server
 
     We inherit and overload the socket server so that we
     can control the client threads as well as have a single
@@ -319,7 +383,7 @@ class ModbusSerialServer(object):
         :param parity: Which kind of parity to use
         :param baudrate: The baud rate to use for the serial device
         :param timeout: The timeout to use for the serial device
-
+        :param ignore_missing_slaves: True to not send errors on a request to a missing slave
         '''
         self.threads = []
         self.decoder = ServerDecoder()
@@ -336,6 +400,7 @@ class ModbusSerialServer(object):
         self.parity   = kwargs.get('parity',   Defaults.Parity)
         self.baudrate = kwargs.get('baudrate', Defaults.Baudrate)
         self.timeout  = kwargs.get('timeout',  Defaults.Timeout)
+        self.ignore_missing_slaves = kwargs.get('ignore_missing_slaves', Defaults.IgnoreMissingSlaves)
         self.socket   = None
         self._connect()
         self.is_running = True
@@ -350,7 +415,7 @@ class ModbusSerialServer(object):
             self.socket = serial.Serial(port=self.device, timeout=self.timeout,
                 bytesize=self.bytesize, stopbits=self.stopbits,
                 baudrate=self.baudrate, parity=self.parity)
-        except serial.SerialException, msg:
+        except serial.SerialException as msg:
             _logger.error(msg)
         return self.socket != None
 
@@ -389,43 +454,48 @@ class ModbusSerialServer(object):
 #---------------------------------------------------------------------------#
 # Creation Factories
 #---------------------------------------------------------------------------#
-def StartTcpServer(context=None, identity=None, address=None):
+def StartTcpServer(context=None, identity=None, address=None, **kwargs):
     ''' A factory to start and run a tcp modbus server
 
     :param context: The ModbusServerContext datastore
     :param identity: An optional identify structure
     :param address: An optional (interface, port) to bind to.
+    :param ignore_missing_slaves: True to not send errors on a request to a missing slave
     '''
     framer = ModbusSocketFramer
-    server = ModbusTcpServer(context, framer, identity, address)
+    server = ModbusTcpServer(context, framer, identity, address, **kwargs)
     server.serve_forever()
 
 
-def StartUdpServer(context=None, identity=None, address=None):
+def StartUdpServer(context=None, identity=None, address=None, **kwargs):
     ''' A factory to start and run a udp modbus server
 
     :param context: The ModbusServerContext datastore
     :param identity: An optional identify structure
     :param address: An optional (interface, port) to bind to.
+    :param framer: The framer to operate with (default ModbusSocketFramer)
+    :param ignore_missing_slaves: True to not send errors on a request to a missing slave
     '''
-    framer = ModbusSocketFramer
-    server = ModbusUdpServer(context, framer, identity, address)
+    framer = kwargs.pop('framer', ModbusSocketFramer)
+    server = ModbusUdpServer(context, framer, identity, address, **kwargs)
     server.serve_forever()
 
 
 def StartSerialServer(context=None, identity=None, **kwargs):
-    ''' A factory to start and run a udp modbus server
+    ''' A factory to start and run a serial modbus server
 
     :param context: The ModbusServerContext datastore
     :param identity: An optional identify structure
+    :param framer: The framer to operate with (default ModbusAsciiFramer)
     :param port: The serial port to attach to
     :param stopbits: The number of stop bits to use
     :param bytesize: The bytesize of the serial messages
     :param parity: Which kind of parity to use
     :param baudrate: The baud rate to use for the serial device
     :param timeout: The timeout to use for the serial device
+    :param ignore_missing_slaves: True to not send errors on a request to a missing slave
     '''
-    framer = ModbusAsciiFramer
+    framer = kwargs.pop('framer', ModbusAsciiFramer)
     server = ModbusSerialServer(context, framer, identity, **kwargs)
     server.serve_forever()
 
